-- Config ----------------
--------------------------
local E = AleaUI
local L = E.L
local UF = E:Module("UnitFrames")

local defaults = {
	enableXP = true,
	enableRep = true,
	enableHonor = true,
	enableArtifact = true,
	minimize = false,
}

E.default_settings.xpbar = defaults


--Bar Height and Width
local barHeight, barWidth = 10, 275.5---Where you want the fame to be anchored
local barSpace = 3
--------AnchorPoint, AnchorTo, RelativePoint, xOffset, yOffset
local Anchor = { "TOP", UIParent, "TOP", -0.5 , -2 }

--Fonts
local showText = true -- Set to false to hide text
local mouseoverText = false -- Set to true to only show text on mouseover
local flags = "OUTLINE"

--Textures
local barTex = E.media.default_bar_texture1
local flatTex = E.media.default_bar_texture2
local colorize 
local overlay 
local CommaValue 

-----------------------------------------------------------
-- Don't edit past here unless you know what your doing! --
-----------------------------------------------------------
-- Tables ----------------
--------------------------
--'Hated', 'Hostile', 'Unfriendly', 'Neutral', 'Friendly', 'Honored', 'Revered', 'Exalted'

local FactionInfo = {
	[1] = {{ 170/255, 70/255,  70/255 }, L['Hated'], "FFaa4646"},
	[2] = {{ 170/255, 70/255,  70/255 }, L['Hostile'], "FFaa4646"},
	[3] = {{ 170/255, 70/255,  70/255 }, L['Unfriendly'], "FFaa4646"},
	[4] = {{ 200/255, 180/255, 100/255 }, L['Neutral'], "FFc8b464"},
	[5] = {{ 75/255,  175/255, 75/255 }, L['Friendly'], "FF4baf4b"},
	[6] = {{ 75/255,  175/255, 75/255 }, L['Honored'], "FF4baf4b"},
	[7] = {{ 75/255,  175/255, 75/255 }, L['Revered'], "FF4baf4b"},
	[8] = {{ 155/255,  255/255, 155/255 }, L['Exalted'],"FF9bff9b"},
}
-- Functions -------------
--------------------------

function CommaValue(amount)
	local formatted = amount
	local k
	while true do  
		formatted, k = string.gsub(formatted, "^(-?%d+)(%d%d%d)", '%1,%2')
		if (k==0) then
			break
		end
	end
	return formatted
end

function colorize(r)
	return FactionInfo[r][3]
end

local function IsMaxLevel()
	if UnitLevel("player") == MAX_PLAYER_LEVEL then
		return true
	end
end

-- Framework -------------
--------------------------

--Prefix for naming frames
local aName = "AleaUI_XPBar"

--Create Background and Border
local backdrop = CreateFrame("Frame", aName.."Backdrop", UIParent)
backdrop:SetHeight(barHeight)
backdrop:SetWidth(barWidth)
backdrop:SetPoint("CENTER", UIParent, "CENTER", 0, 0)
backdrop:SetBackdrop({
	bgFile = barTex, 
	edgeFile = barTex, 
	tile = false, tileSize = 0, edgeSize = 1, 
	insets = { left = -1, right = -1, top = -1, bottom = -1}
})
backdrop:SetBackdropColor(0, 0, 0)
backdrop:SetBackdropBorderColor(.2, .2, .2, 0)

overlay = backdrop:CreateTexture(nil, "BORDER", backdrop)
overlay:ClearAllPoints()
overlay:SetPoint("TOPLEFT", backdrop, "TOPLEFT", 1, -1)
overlay:SetPoint("BOTTOMRIGHT", backdrop, "BOTTOMRIGHT", -1, 1)
overlay:SetTexture(barTex)
overlay:SetVertexColor(.1,.1,.1)

--Create xp status bar
local xpBar = CreateFrame("StatusBar",  aName.."xpBar", backdrop, "TextStatusBar")
xpBar:SetWidth(barWidth)
xpBar:SetHeight(GetWatchedFactionInfo() and (barHeight) or barHeight)
xpBar:SetPoint("TOP", backdrop,"TOP", 0, 0)
xpBar:SetStatusBarTexture(barTex)local _, class = UnitClass("player") local color = RAID_CLASS_COLORS[class] local r, g, b = color.r, color.g, color.b 
xpBar:SetStatusBarColor(r, g, b)

xpBar.Text = xpBar:CreateFontString(aName.."ArtifactBarText", "OVERLAY")
xpBar.Text:SetFont(AleaUI.media.default_font, AleaUI.media.default_font_size, flags)
xpBar.Text:SetPoint("CENTER", xpBar, "CENTER", 0, 1)
xpBar.Text:SetAlpha(1)

--Create Rested XP Status Bar
local restedxpBar = CreateFrame("StatusBar", aName.."restedxpBar", backdrop, "TextStatusBar")
restedxpBar:SetHeight(GetWatchedFactionInfo() and (barHeight) or barHeight)
restedxpBar:SetWidth(barWidth)
restedxpBar:SetPoint("TOP", backdrop,"TOP", 0, 0)
restedxpBar:SetStatusBarTexture(barTex)
restedxpBar:Hide()

local artifactBar = CreateFrame("StatusBar",  aName.."ArtifactBar", backdrop, "TextStatusBar")
artifactBar:SetWidth(barWidth)
artifactBar:SetHeight(GetWatchedFactionInfo() and (barHeight) or barHeight)
artifactBar:SetPoint("BOTTOM",backdrop,"BOTTOM", 0, 0)
artifactBar:SetStatusBarTexture(barTex)
artifactBar:SetStatusBarColor(0.8, 0.8, 0)

artifactBar.Text = artifactBar:CreateFontString(aName.."ArtifactBarText", "OVERLAY")
artifactBar.Text:SetFont(AleaUI.media.default_font, AleaUI.media.default_font_size, flags)
artifactBar.Text:SetPoint("CENTER", artifactBar, "CENTER", 0, 1)
artifactBar.Text:SetAlpha(1)

artifactBar.background1 = artifactBar:CreateTexture(nil, 'BACKGROUND')
artifactBar.background1:SetColorTexture(0, 0, 0, 1)
artifactBar.background1:SetPoint('TOPLEFT', -1, 1)
artifactBar.background1:SetPoint('BOTTOMRIGHT', 1, -1)

artifactBar.background2 = artifactBar:CreateTexture(nil, 'BORDER')
artifactBar.background2:SetTexture(barTex)
artifactBar.background2:SetVertexColor(0.3, 0.3, 0, 1)
artifactBar.background2:SetPoint('TOPLEFT', 0, 0)
artifactBar.background2:SetPoint('BOTTOMRIGHT', 0, 0)

local honorBar = CreateFrame("StatusBar",  aName.."HonorBar", backdrop, "TextStatusBar")
honorBar:SetWidth(barWidth)
honorBar:SetHeight(GetWatchedFactionInfo() and (barHeight) or barHeight)
honorBar:SetPoint("BOTTOM",backdrop,"BOTTOM", 0, 0)
honorBar:SetStatusBarTexture(barTex)
honorBar:SetStatusBarColor(255/255, 156/255, 0)

honorBar.Text = honorBar:CreateFontString(aName.."ArtifactBarText", "OVERLAY")
honorBar.Text:SetFont(AleaUI.media.default_font, AleaUI.media.default_font_size, flags)
honorBar.Text:SetPoint("CENTER", honorBar, "CENTER", 0, 1)
honorBar.Text:SetAlpha(1)

honorBar.background1 = honorBar:CreateTexture(nil, 'BACKGROUND')
honorBar.background1:SetColorTexture(0, 0, 0, 1)
honorBar.background1:SetPoint('TOPLEFT', -1, 1)
honorBar.background1:SetPoint('BOTTOMRIGHT', 1, -1)

honorBar.background2 = honorBar:CreateTexture(nil, 'BORDER')
honorBar.background2:SetTexture(barTex)
honorBar.background2:SetVertexColor(255/255*0.3, 156/255*0.3, 0, 1)
honorBar.background2:SetPoint('TOPLEFT', 0, 0)
honorBar.background2:SetPoint('BOTTOMRIGHT', 0, 0)


--Create reputation status bar (Only used if not max level)
local repBar = CreateFrame("StatusBar", aName.."repBar", backdrop, "TextStatusBar")
repBar:SetWidth(barWidth)
repBar:SetHeight(IsMaxLevel() and barHeight-0 or 0)
repBar:SetPoint("BOTTOM",backdrop,"BOTTOM", 0, 0)
repBar:SetStatusBarTexture(barTex)

repBar.Text = repBar:CreateFontString(aName.."ArtifactBarText", "OVERLAY")
repBar.Text:SetFont(AleaUI.media.default_font, AleaUI.media.default_font_size, flags)
repBar.Text:SetPoint("CENTER", repBar, "CENTER", 0, 1)
repBar.Text:SetAlpha(1)

repBar.background1 = repBar:CreateTexture(nil, 'BACKGROUND')
repBar.background1:SetColorTexture(0, 0, 0, 1)
repBar.background1:SetPoint('TOPLEFT', -1, 1)
repBar.background1:SetPoint('BOTTOMRIGHT', 1, -1)

repBar.background2 = repBar:CreateTexture(nil, 'BORDER')
repBar.background2:SetTexture(barTex)
repBar.background2:SetVertexColor(0.3, 0.3, 0, 1)
repBar.background2:SetPoint('TOPLEFT', 0, 0)
repBar.background2:SetPoint('BOTTOMRIGHT', 0, 0)

--Create frame used for mouseover, clicks, and text
local mouseFrame = CreateFrame("Frame", aName.."mouseFrame", backdrop)
mouseFrame:SetPoint('TOP', backdrop, 'TOP', 0, 0)
mouseFrame:EnableMouse(true)

-- InActiveBattlefield() or IsInActiveWorldPVP()

-- Level Left Text

local LevelText = xpBar:CreateFontString(aName.."LevelText", "OVERLAY")
LevelText:SetFont(AleaUI.media.default_font, AleaUI.media.default_font_size, flags)
LevelText:SetPoint("RIGHT", xpBar, "LEFT", 0, -1)



--Set all frame levels (easier to see if organized this way)
backdrop:SetFrameLevel(0)
restedxpBar:SetFrameLevel(1)
repBar:SetFrameLevel(2)
xpBar:SetFrameLevel(2)
artifactBar:SetFrameLevel(2)
honorBar:SetFrameLevel(2)
mouseFrame:SetFrameLevel(3)

local attept = 3

local function updateStatus()
	local XP, maxXP = UnitXP("player"), UnitXPMax("player")
	if maxXP == 0 then return end

	local restXP = GetXPExhaustion()
	local percXP = floor(XP/maxXP*100)
	
	local HeightStep = 0
	
	backdrop:SetHeight(barHeight)
	xpBar:SetHeight(barHeight)
	repBar:SetHeight(barHeight)
	artifactBar:SetHeight(barHeight)
	restedxpBar:SetHeight(barHeight)	
	honorBar:SetHeight(barHeight)	
	
	xpBar.Text:SetShown(not E.db.xpbar.minimize)
	repBar.Text:SetShown(not E.db.xpbar.minimize)
	artifactBar.Text:SetShown(not E.db.xpbar.minimize)
	honorBar.Text:SetShown(not E.db.xpbar.minimize)
	
	if IsMaxLevel() then
		xpBar:Hide()
		restedxpBar:Hide()

		if not GetWatchedFactionInfo() and not HasArtifactEquipped() then
			backdrop:Hide()
		else
			backdrop:Show()
		end
		
		LevelText:SetText('')		
	elseif E.db.xpbar.enableXP then
		xpBar:Show()
		restedxpBar:Show()
		
		HeightStep = HeightStep + barHeight + barSpace
		
		if level ~= MAX_PLAYER_LEVEL then
			LevelText:SetText('|cff4baf4c'..UnitLevel('player'))
		end

		xpBar:SetMinMaxValues(min(0, XP), maxXP)
		xpBar:SetValue(XP)
			
		if restXP then
			restedxpBar:Show()
			
			local _, class = UnitClass("player") 
			local color = RAID_CLASS_COLORS[class] 
			local r, g, b = color.r, color.g, color.b	
			
			restedxpBar:SetStatusBarColor(r,g,b, 0.40)
			restedxpBar:SetMinMaxValues(min(0, XP), maxXP)
			restedxpBar:SetValue(XP+restXP)
			xpBar.Text:SetText(format("%s/%s (%s%%|cffb3e1ff+%d%%|r)",UF:ShortValue(XP),UF:ShortValue(maxXP), percXP, restXP/maxXP*100))
		else
			restedxpBar:Hide()
			xpBar.Text:SetText(format("%s/%s (%s%%)", UF:ShortValue(XP), UF:ShortValue(maxXP), percXP))
		end
	end
	
	if HasArtifactEquipped and HasArtifactEquipped() and E.db.xpbar.enableArtifact then
	
		local itemID, altItemID, name, icon, totalXP, pointsSpent, quality, artifactAppearanceID, appearanceModID, itemAppearanceID, altItemAppearanceID, altOnTop = C_ArtifactUI.GetEquippedArtifactInfo();
		local numPointsAvailableToSpend, xp, xpForNextPoint = MainMenuBar_GetNumArtifactTraitsPurchasableFromXP(pointsSpent, totalXP);
		
		artifactBar:Show()
		artifactBar.xp = xp;
		artifactBar.totalXP = totalXP;
		artifactBar.xpForNextPoint = xpForNextPoint;
		artifactBar.numPointsAvailableToSpend = numPointsAvailableToSpend;
		artifactBar:SetMinMaxValues(0, xpForNextPoint)
		artifactBar:SetValue(xp)
		
		artifactBar.Text:SetText(format('%d/%d (%d%%)', xp, xpForNextPoint, ( xp/xpForNextPoint*100) ))
		
		artifactBar:SetPoint("BOTTOM",backdrop,"BOTTOM", 0, -HeightStep)
		
		HeightStep = HeightStep + barHeight + barSpace
	else
		if UnitLevel('player') == 110 and attept > 0 then
			attept = attept - 1
			C_Timer.After(1, updateStatus)
		end
		
		artifactBar:Hide()
	end
	
	if E.db.xpbar.enableHonor then
		
		local current = UnitHonor("player");
		local max = UnitHonorMax("player");

		local level = UnitHonorLevel("player");
        local levelmax = GetMaxPlayerHonorLevel();
		
		honorBar:Show()
		
		if (level == levelmax) then
			honorBar:SetMinMaxValues(0, 1)
			honorBar:SetValue(1)
			
			honorBar.Text:SetText('')
		else
		
			honorBar:SetMinMaxValues(0, max)
			honorBar:SetValue(current)
			
			honorBar.Text:SetText(format('%d/%d (%d%%)', current, max, (current/max*100) ))
		
		end
		
		honorBar:SetPoint("BOTTOM",backdrop,"BOTTOM", 0, -HeightStep)
		
		HeightStep = HeightStep + barHeight + barSpace
	else
		honorBar:Hide()
	end
	
	if GetWatchedFactionInfo() and E.db.xpbar.enableRep then
		local name, rank, minRep, maxRep, value = GetWatchedFactionInfo()
		repBar:SetMinMaxValues(minRep, maxRep)
		repBar:SetValue(value)
		repBar:SetStatusBarColor(unpack(FactionInfo[rank][1]))
		
		repBar.Text:SetText(format("%d / %d (%d%%)", value-minRep, maxRep-minRep, (value-minRep)/(maxRep-minRep)*100))
		repBar:Show()
		
		repBar:SetPoint("BOTTOM",backdrop,"BOTTOM", 0, -HeightStep)
		HeightStep = HeightStep + barHeight + barSpace
	else
		repBar:Hide()
	end
	
	--Setup Exp Tooltip
	
	mouseFrame:SetHeight(HeightStep)
	mouseFrame:SetWidth(barWidth)

	mouseFrame:SetScript("OnEnter", function()
		GameTooltip:SetOwner(mouseFrame, "ANCHOR_BOTTOMLEFT", -3, barHeight)
		GameTooltip:ClearLines()
		if not IsMaxLevel() then
			GameTooltip:AddLine(XP..":")
			GameTooltip:AddLine(string.format(L['Value']..': %s/%s (%d%%)', CommaValue(XP), CommaValue(maxXP), (XP/maxXP)*100))
			GameTooltip:AddLine(string.format(L['VALUE_LEFT']..': %s', CommaValue(maxXP-XP)))
			if restXP then
				GameTooltip:AddLine(string.format('|cffb3e1ff'..L['Rested']..': %s (%d%%)', CommaValue(restXP), restXP/maxXP*100))
			end
		end
		
		if HasArtifactEquipped and HasArtifactEquipped() then
			if not IsMaxLevel() then GameTooltip:AddLine(" ") end
			local itemID, altItemID, name = C_ArtifactUI.GetEquippedArtifactInfo();
			GameTooltip:AddLine(L['Artifact']..": ".. name)	
			GameTooltip:AddLine(string.format(L['Current']..': %s/%s', artifactBar.xp, artifactBar.xpForNextPoint))
			GameTooltip:AddLine(string.format(L['VALUE_LEFT']..': %s', artifactBar.xpForNextPoint-artifactBar.xp))
		end
		
		if GetWatchedFactionInfo() then
			local name, rank, min, max, value = GetWatchedFactionInfo()
			if HasArtifactEquipped() then 
				GameTooltip:AddLine(" ") 
			elseif not IsMaxLevel() then 
				GameTooltip:AddLine(" ")
			end
			GameTooltip:AddLine(string.format(L['Reputation']..': %s', name))
			GameTooltip:AddLine(string.format(L['Attitude']..': |c'..colorize(rank)..'%s|r', FactionInfo[rank][2]))
			GameTooltip:AddLine(string.format(L['Value']..': %s/%s (%d%%)', CommaValue(value-min), CommaValue(max-min), (value-min)/(max-min)*100))
			GameTooltip:AddLine(string.format(L['VALUE_LEFT']..': %s', CommaValue(max-value)))
		end
		
		GameTooltip:Show()
	end)
	mouseFrame:SetScript("OnLeave", function()
		GameTooltip:Hide()
	end)
	
end

-- Event Stuff -----------
--------------------------
local frame = CreateFrame("Frame",nil,UIParent)
--Event handling
frame:RegisterEvent("PLAYER_LEVEL_UP")
frame:RegisterEvent("PLAYER_XP_UPDATE")
frame:RegisterEvent("UPDATE_EXHAUSTION")
frame:RegisterEvent("CHAT_MSG_COMBAT_FACTION_CHANGE")
frame:RegisterEvent("UPDATE_FACTION")
frame:RegisterEvent("PLAYER_ENTERING_WORLD")
frame:RegisterEvent("ARTIFACT_XP_UPDATE")
frame:RegisterEvent("ARTIFACT_UPDATE")
frame:RegisterEvent("ARTIFACT_CLOSE")
frame:RegisterUnitEvent("UNIT_INVENTORY_CHANGED", 'player')

frame:RegisterEvent("HONOR_XP_UPDATE");
frame:RegisterEvent("HONOR_LEVEL_UPDATE");
frame:RegisterEvent("PLAYER_UPDATE_RESTING");
	
	
frame:SetScript("OnEvent", updateStatus)

local function InitXPBar()
	AleaUI:Mover(backdrop, "xpbarFrame", 276, 10)
	
	if E.db.xpbar.minimize then
		barHeight = 4
		barSpace = 1
	else
		barHeight = 10
		barSpace = 3
	end
	
	updateStatus()
	
	E.GUI.args.general.args.xpbar = {
		name = L['Xp, honor, artifact and reputation bars'],
		order = 5,
		type = 'group',
		embend = true,
		args = {},
	}

	E.GUI.args.general.args.xpbar.args.minimize = {
		name = L['Minimize'],
		order = 1,
		type = 'toggle', 
		set = function() 
			E.db.xpbar.minimize = not E.db.xpbar.minimize
			
			if E.db.xpbar.minimize then
				barHeight = 4
			else
				barHeight = 10
			end
			
			updateStatus()
		end,
		get = function()
			return E.db.xpbar.minimize
		end,
	}
	
	E.GUI.args.general.args.xpbar.args.unlock = {
		name = L['Unlock'],
		order = 1.1,
		type = "execute",
		set = function(self, value)
			E:UnlockMover("xpbarFrame") 
		end,
		get = function(self)end
	}
	
	E.GUI.args.general.args.xpbar.args.EnableXP = {
		name = L['Enable xp bar'],
		order = 2,
		type = 'toggle',
		set = function()
			E.db.xpbar.enableXP = not E.db.xpbar.enableXP
		end,
		get = function()
			return E.db.xpbar.enableXP
		end,
	}
	
	E.GUI.args.general.args.xpbar.args.EnableRep = {
		name = L['Enable reputation bar'],
		order = 3,
		type = 'toggle',
		set = function()
			E.db.xpbar.enableRep = not E.db.xpbar.enableRep
			updateStatus()
		end,
		get = function()
			return E.db.xpbar.enableRep
		end,
	}
	
	E.GUI.args.general.args.xpbar.args.EnableHonor = {
		name = L['Enable honor bar'],
		order = 4,
		type = 'toggle',
		set = function()
			E.db.xpbar.enableHonor = not E.db.xpbar.enableHonor
			updateStatus()
		end,
		get = function()
			return E.db.xpbar.enableHonor
		end,
	}

	E.GUI.args.general.args.xpbar.args.EnableArtifact = {
		name = L['Enable artifact bar'],
		order = 5,
		type = 'toggle',
		set = function()
			E.db.xpbar.enableArtifact = not E.db.xpbar.enableArtifact
			updateStatus()
		end,
		get = function()
			return E.db.xpbar.enableArtifact
		end,
	}

end

AleaUI:OnInit(InitXPBar)